/*-
 * Copyright (c) 2004 Gleb Smirnoff <glebius@cell.sick.ru>
 * Copyright (c) 2001-2003 Roman V. Palagin <romanp@unshadow.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by Gleb Smirnoff and
 * contributors.
 * 4. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: kernel.h,v 1.7.2.2 2004/08/10 09:09:27 glebius Exp $
 */

/* Parse the info structure */
#define	NG_NETFLOW_INFO_TYPE	{			\
	{ "packets",	&ng_parse_uint32_type },	\
	{ "records",	&ng_parse_uint32_type },	\
	{ "Inactive timeout",	&ng_parse_uint32_type },\
	{ "Active timeout",	&ng_parse_uint32_type },\
	{ NULL }					\
}

/* Parse the ifinfo structure */
#define NG_NETFLOW_IFINFO_TYPE	{			\
	{ "index",	&ng_parse_uint16_type },	\
	{ "packets",	&ng_parse_uint32_type },	\
	{ "data link type", &ng_parse_uint8_type },	\
	{ NULL }					\
}

/* Parse the setdlt structure */
#define	NG_NETFLOW_SETDLT_TYPE {			\
	{ "iface",	&ng_parse_uint16_type },	\
	{ "dlt",	&ng_parse_uint8_type  },	\
	{ NULL }					\
}

/* Parse the setifindex structure */
#define	NG_NETFLOW_SETIFINDEX_TYPE {			\
	{ "iface",	&ng_parse_uint16_type },	\
	{ "index",	&ng_parse_uint16_type },	\
	{ NULL }					\
}

/* Parse the settimeouts structure */
#define NG_NETFLOW_SETTIMEOUTS_TYPE {			\
	{ "inactive",	&ng_parse_uint32_type },	\
	{ "active",	&ng_parse_uint32_type },	\
	{ NULL }					\
}

struct ng_netflow_iface {
	hook_p		hook;		/* NULL when disconnected */
	struct ng_netflow_ifinfo	info;
};

typedef struct ng_netflow_iface *iface_p;
typedef struct ng_netflow_ifinfo *ifinfo_p;

struct netflow {
	node_p			node;		/* link to the node itself */

	/* incoming traffic comes in here */
	struct ng_netflow_iface	ifaces[NG_NETFLOW_MAXIFACES];
	/* export data goes there */
	hook_p  		export;

	struct ng_netflow_info	info;
	long			last_expire;

	struct flow_hash  	*hash;
	LIST_HEAD(, flow_chunk) cache;

	u_int32_t		flow_seq;
	/* This two MUST go together, since export_send() points
	 * to first one */
	struct netflow_v5_header header;
	struct netflow_v5_record r[NETFLOW_V5_MAX_RECORDS];
};

typedef struct netflow *priv_p;

struct flow_hash {
	LIST_HEAD( ,flow_entry) head;
};

/* Make sure packet large enough to contain len bytes */
#define	CHECK_MLEN(m, length)	((m)->m_pkthdr.len < (length))
#define CHECK_PULLUP(m, length)	((m)->m_len < (length) && \
				(((m) = m_pullup((m),(length))) == NULL))

#define ERROUT(x)	{ error = (x); goto done; }

/* Prototypes for netflow.c */
int hash_init(priv_p);
void hash_flush(priv_p);
int flow_add(priv_p, struct mbuf **, iface_p);
int flow_show(priv_p, struct ng_mesg *);
int expire(priv_p );
