/*-
 * Copyright (c) 2004 Gleb Smirnoff <glebius@cell.sick.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by Gleb Smirnoff and
 * contributors.
 * 4. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: netflow.h,v 1.3.2.1 2004/06/22 08:47:54 glebius Exp $
 */

/* netflow timeouts in seconds */

#define	ACTIVE_TIMEOUT		(30*60)	/* maximum flow lifetime is 30 min */
#define	INACTIVE_TIMEOUT	15

#define	MAX_FLOW_CACHE		65535

/* More info can be found in Cisco's white paper:
 * http://www.cisco.com/warp/public/cc/pd/iosw/ioft/neflct/tech/napps_wp.htm
 */

#define NETFLOW_V1 1
#define NETFLOW_V5 5

struct netflow_v1_header
{
  u_int16_t version;	/* NetFlow version */
  u_int16_t count;	/* Number of records in flow */
  u_int32_t sys_uptime;	/* System uptime */
  u_int32_t unix_secs;	/* Current seconds since 0000 UTC 1970 */
  u_int32_t unix_nsecs;	/* Remaining nanoseconds since 0000 UTC 1970 */
} __attribute__((__packed__));

struct netflow_v5_header
{
  u_int16_t version;	/* NetFlow version */
  u_int16_t count;	/* Number of records in flow */
  u_int32_t sys_uptime;	/* System uptime */
  u_int32_t unix_secs;	/* Current seconds since 0000 UTC 1970 */
  u_int32_t unix_nsecs;	/* Remaining nanoseconds since 0000 UTC 1970 */
  u_int32_t flow_seq;	/* Sequence number of the first record */
  u_int8_t engine_type;	/* Type of flow switching engine (RP,VIP,etc.) */
  u_int8_t engine_id;	/* Slot number of the flow switching engine */
  u_int16_t pad;	/* pad to word boundary */
} __attribute__((__packed__));

struct netflow_v1_record
{
  u_int32_t src_addr;	/* Source IP address */
  u_int32_t dst_addr;	/* Destination IP address */
  u_int32_t next_hop;	/* Next hop IP address */
  u_int16_t in_ifx;	/* Source interface index */
  u_int16_t out_ifx;	/* Destination interface index */
  u_int32_t packets;	/* Number of packets in a flow */
  u_int32_t octets;	/* Number of octets in a flow */
  u_int32_t first;	/* System uptime at start of a flow */
  u_int32_t last;	/* System uptime at end of a flow */
  u_int16_t s_port;	/* Source port */
  u_int16_t d_port;	/* Destination port */
  u_int16_t pad1;	/* pad to word boundary */
  u_int8_t prot;	/* IP protocol */
  u_int8_t tos;		/* IP type of service */
  u_int8_t flags;	/* Cumulative OR of tcp flags */
  u_int8_t pad2;	/* pad to word boundary */
  u_int16_t pad3;	/* pad to word boundary */
  u_int8_t reserved[5];	/* reserved for future use */
} __attribute__((__packed__));

struct netflow_v5_record
{
  u_int32_t src_addr;	/* Source IP address */
  u_int32_t dst_addr;	/* Destination IP address */
  u_int32_t next_hop;	/* Next hop IP address */
  u_int16_t i_ifx;	/* Source interface index */
  u_int16_t o_ifx;	/* Destination interface index */
  u_int32_t packets;	/* Number of packets in a flow */
  u_int32_t octets;	/* Number of octets in a flow */
  u_int32_t first;	/* System uptime at start of a flow */
  u_int32_t last;	/* System uptime at end of a flow */
  u_int16_t s_port;	/* Source port */
  u_int16_t d_port;	/* Destination port */
  u_int8_t pad1;	/* pad to word boundary */
  u_int8_t flags;	/* Cumulative OR of tcp flags */
  u_int8_t prot;	/* IP protocol */
  u_int8_t tos;		/* IP type of service */
  u_int16_t dst_as;	/* dst peer/origin Autonomous System */
  u_int16_t src_as;	/* src peer/origin Autonomous System */
  u_int8_t dst_mask;	/* destination route's mask bits */
  u_int8_t src_mask;	/* source route's mask bits */
  u_int16_t pad2;	/* pad to word boundary */
} __attribute__((__packed__));

#define NETFLOW_V1_MAX_RECORDS 24
#define NETFLOW_V5_MAX_RECORDS 30

#define NETFLOW_V1_MAX_SIZE (sizeof(netflow_v1_header)+ \
			     sizeof(netflow_v1_record)*NETFLOW_V1_MAX_RECORDS)
#define NETFLOW_V5_MAX_SIZE (sizeof(netflow_v5_header)+ \
			     sizeof(netflow_v5_record)*NETFLOW_V5_MAX_RECORDS)
