/*-
 * Copyright (c) 2004 Gleb Smirnoff <glebius@cell.sick.ru>
 * Copyright (c) 2001-2003 Roman V. Palagin <romanp@unshadow.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * This product includes software developed by Gleb Smirnoff and
 * contributors.
 * 4. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: ng_netflow.h,v 1.9.2.4 2004/08/10 10:19:47 glebius Exp $
 */

#define NG_NETFLOW_NODE_TYPE	"netflow"
#define NGM_NETFLOW_COOKIE	1092127674

#define	NG_NETFLOW_MAXIFACES	32

/* Hook names */

#define	NG_NETFLOW_HOOK_DATA	"iface"
#define NG_NETFLOW_HOOK_EXPORT	"export"

/* Netgraph commands understood by netflow node */
enum {
    NGM_NETFLOW_INFO = 1,	/* get node info */
    NGM_NETFLOW_IFINFO,		/* get iface info */
    NGM_NETFLOW_SHOW,		/* show ip cache flow */
    NGM_NETFLOW_SETDLT,		/* set data-link type */	
    NGM_NETFLOW_SETIFINDEX, 	/* set interface index */
    NGM_NETFLOW_SETTIMEOUTS, 	/* set active/inactive flow timeouts */
};

/* This structure is returned by the NGM_NETFLOW_INFO message */
struct ng_netflow_info {
	u_int32_t   	packets;	/* total number of accounted packets */
	u_int32_t	records;	/* total number of hash records */
	u_int32_t	inactive_timeout;	/* flow inactive timeout */
	u_int32_t	active_timeout;		/* flow active timeout */
};

/* This structure is returned by the NGM_NETFLOW_IFINFO message */
struct ng_netflow_ifinfo {
	u_int16_t	if_index;	/* connected iface index */
	u_int32_t       if_packets;	/* number of packets for this iface */
	u_int8_t	if_dlt;		/* Data Link Type, DLT_XXX */
};

#define	MAXDLTNAMELEN	20

/* This structure is passed to NGM_NETFLOW_SETDLT message */
struct ng_netflow_setdlt {
	u_int16_t iface;		/* which iface dlt change */
	u_int8_t  dlt;			/* DLT_XXX from bpf.h */
};

/* This structure is passed to NGM_NETFLOW_SETIFINDEX */
struct ng_netflow_setifindex {
	u_int16_t iface;		/* which iface index change */
	u_int16_t index;		/* new index */
};

/* This structure is passed to NGM_NETFLOW_SETTIMEOUTS */
struct ng_netflow_settimeouts {
	u_int32_t	inactive_timeout;	/* flow inactive timeout */
	u_int32_t	active_timeout;		/* flow active timeout */
};

/* This is unique data, which identifies flow */
struct flow_rec {
	struct in_addr	r_src;
	struct in_addr	r_dst;
	union {
		struct {
			u_int16_t	s_port;	/* source TCP/UDP port */
			u_int16_t	d_port; /* destination TCP/UDP port */
		} dir;
		u_int32_t both;
	} ports;
	union {
		struct {
			u_char		prot;	/* IP protocol */
			u_char		tos;	/* IP TOS */
			u_int16_t	i_ifx;	/* input interface index */
		} i;
		u_int32_t all;
	} misc;
};

#define	r_ip_p	misc.i.prot
#define	r_tos	misc.i.tos
#define	r_i_ifx	misc.i.i_ifx
#define r_misc	misc.all
#define r_ports	ports.both
#define r_sport	ports.dir.s_port
#define r_dport	ports.dir.d_port
	
/* Flow stats entry */
struct flow_entry {
	struct flow_rec		r;
	struct in_addr		next_hop;
	u_int16_t		fle_o_ifx;	/* output interface index */
#define				fle_i_ifx		r.misc.i.i_ifx
	u_int8_t		dst_mask;	/* destination route's mask bits */
	u_int8_t		src_mask;	/* source route's mask bits */
	/* XXX: If "packets" field is zero, then flow entry is free */
	u_long			packets;
	u_long			bytes;
	long			first;		/* uptime on first packet */
	long			last;		/* uptime on last packet */
	u_char			tcp_flags;	/* cumulative OR */
	struct flow_chunk	*my_chunk;
	LIST_ENTRY(flow_entry) next;
};

/* Chunk of memory for storing flow cache */
#define	CHUNK_SIZE	1024
struct flow_chunk {
	struct flow_entry	chunk[CHUNK_SIZE];
	u_int32_t		flows;
	LIST_ENTRY(flow_chunk)	entries;
};

/*
 * How many flow records we will transfer at once
 * without overflowing socket receive buffer
 */
#define NREC_AT_ONCE		1000
#define NGRESP_SIZE		(sizeof(struct flows) + NREC_AT_ONCE * sizeof(struct flow_entry))
#define SORCVBUF_SIZE		(NGRESP_SIZE + 2 * sizeof(struct ng_mesg))

/* This struct is returned to userland, when "show cache ip flow" */
struct flows {
	u_int32_t		nentries;
	struct flow_entry	entries[0];
};
